%% QMG 2.0 test 7: a circle with two elliptic holes and a slit.


global interactive
%% Make an approximation to a circle using 6 Bezier cubic arcs.

numseg = 6;
[verts, scrap] = gm_circ_approx(0, 2*pi, numseg);

[numvert,scrap] = size(verts);
verts = verts(1:3*numseg,:);
codes = kron(ones(numseg,1),[2;3;3]);
codes(1) = 0;

c0 = gm_cpoly(verts,codes);

%% Make two elliptic holes.

hole1 = gmapply([.2, 0, 0; 0,.1,-.5], c0);
c = gmcavity(c0,hole1);
hole2 = gmapply([.2, 0, 0; 0, .1, .5], c0);
c = gmcavity(c,hole2);

%% Make a slit by modifying the brep directly.


%% Insert two new control points at (-.6,-.1) and (.6,.1).
[scrap,numcp] = size(c{4});

c{4}(:,numcp:numcp+1) = [.6,-.6;.1,-.1];

%% Insert two new vertices at these points.

[scrap,numv] = size(c{5});

c{5}(:,numv:numv+1) = {
'newv1','newv2'
 {}, {}
 {}, {}
 {}, {}
 {'vertex';[];numcp},{'vertex';[];numcp+1}};

%% Insert a new edge connecting those vertices.

[scrap,nume] = size(c{6});

c{6}(:,nume) = {
'newe1'
 {}
 {'newv1','newv2'}
 {}
 {'bezier_curve';1;[numcp,numcp+1]}};

%% Insert the new edge as a repeated boundary of the region.

oldbsize = length(c{7}{2,0}); 
c{7}{2,0}{oldbsize} = 'newe1';
c{7}{2,0}{oldbsize+1} = 'newe1';


c_color = gmrndcolor(c);

if length(interactive) > 0
  gmviz(c_color), gmshowcolor(c_color)
end

%% generate a mesh.  Elements of size about .2.

show = 0;
if length(interactive) > 0
  show = 1;
end

m = gmmeshgen(c_color,'size','(const .2)', 'show', show);


%% Display the mesh
if length(interactive) > 0
  gmviz(m)
end

[c_color2, m2] = gmdouble(c_color, {'*'}, m);

%% Check the mesh.

asp = gmchecktri(c_color2,m2);


%% Default neumann BC du/dn=0 on all faces except the two
%% holes that get Dirichlet conditions.

for j = 0 : nume
  if length(findstr('cav1',double(c_color2{6}{0,j})))
    [npv,scrap] = size(c_color2{6}(1,j));
    c_color2{6}{1,j}(:,npv) = {'bc'; '(d (const 1))'};
  end
  if length(findstr('cav2',double(c_color2{6}{0,j})))
    [npv,scrap] = size(c_color2{6}(1,j));
    c_color2{6}{1,j}(:,npv) = {'bc'; '(d (const 2))'};
  end
end

u = gmfem(c_color2, m2);

if length(interactive) > 0
  gmplot(m2, u)
end

global aspprod
global meshsizesum
[scrap,numvtx] = size(m2{4});

if length(aspprod) > 0
  aspprod = aspprod * asp;
  meshsizesum = meshsizesum + numvtx;
end


% ------------------------------------------------------------------
% Copyright (c) 1999 by Cornell University.  All rights reserved
% See the accompanying file 'Copyright' for authorship information,
% the terms of the license governing this software, and disclaimers
% concerning this software.
% ------------------------------------------------------------------
% This file is part of the QMG software.  
% Version 2.0 of QMG, release date September 3, 1999.
% ------------------------------------------------------------------
