// ------------------------------------------------------------------
// qlifter.h
//
// This file contains a class for lifting parameter values 
// from low-dimensional subfaces.
// ------------------------------------------------------------------
// Author: Stephen A. Vavasis
// Copyright (c) 1999 by Cornell University.  All rights reserved.
// 
// See the accompanying file 'Copyright' for authorship information,
// the terms of the license governing this software, and disclaimers
// concerning this software.
// ------------------------------------------------------------------
// This file is part of the QMG software.  
// Version 2.0 of QMG, release date September 3, 1999.
// ------------------------------------------------------------------

#include "qbrep.h"


namespace QMG {
  class Brep_Parameter_Lifter;
}


class QMG::Brep_Parameter_Lifter {
private:
  const QMG::Brep& b_;

  struct Bdry_CtlPoint_Key {
    int bdry_dim;
    Brep::ControlPointIndex bdry_pts[2];
    bool operator<(const Bdry_CtlPoint_Key& o) const;
  };

  struct Patch_Address {
    Brep::Face_Spec fspec;
    Brep::PatchIndex patchind;
    // Interpretation of lift_coord:
    // 0 = set to 0
    // 1 = set to 1
    // 2 = set to param[0]
    // 3 = set to 1-param[0]
    int lift_coord[2];
  };
    
  multimap<Bdry_CtlPoint_Key, Patch_Address> lookup_map_;

  void insert_(int dim,
               Brep::ControlPointIndex cp1,
               Brep::ControlPointIndex cp2,
               int lift1, 
               int lift2,
               const Brep::Face_Spec& fspec,
               Brep::PatchIndex& patchind);

public:
  explicit Brep_Parameter_Lifter(const Brep& b);
  ~Brep_Parameter_Lifter() { }
  pair<Point, Brep::PatchIndex>
   lift_parameters(const Brep::Face_Spec& orig_fspec,
                   Brep::PatchIndex orig_patchind,
                   const Point& orig_paramcoord,
                   const Brep::Face_Spec& target_fspec) const;
};
