// ------------------------------------------------------------------
// gmchecktri.cpp
//
// This file contains the driver for gmchecktri, which 
// checks that a mesh is valid for a brep.
// ------------------------------------------------------------------
// Author: Stephen A. Vavasis
// Copyright (c) 1999 by Cornell University.  All rights reserved.
// 
// See the accompanying file 'Copyright' for authorship information,
// the terms of the license governing this software, and disclaimers
// concerning this software.
// ------------------------------------------------------------------
// This file is part of the QMG software.  
// Version 2.0 of QMG, release date September 3, 1999.
// ------------------------------------------------------------------

#include "qbrep.h"
#include "qsimpcomp.h"
#include "qfrontend.h"

#define GM_ROUTINE_NAME gmchecktri
#define GM_ROUTINE_NAME_Q "gmchecktri"

namespace QMG {
  extern Real checktri(const Brep& brep,
    const SimpComplex& simpcomp,
    ostream& os,
    int checko,
    double tol);

}

namespace {
  static void worker(const QMG::FrontEnd::ArgValType& argvalhandle,
    QMG::FrontEnd::ReturnValType& returnvalhandle,
    QMG::FrontEnd::Interpreter& interp) {
    
    using namespace QMG;
    using namespace QMG::FrontEnd;
    
    argvalhandle.verify_nargin(2, 4, GM_ROUTINE_NAME_Q);
    returnvalhandle.verify_nargout(0, 1, GM_ROUTINE_NAME_Q);
    int checko = (argvalhandle.nargin() > 2)? 
      argvalhandle.get_int(2) : 1;
    double tol = (argvalhandle.nargin() > 3)?
      argvalhandle.get_double(3) : default_tol(interp);

    InterpOStream os(interp);

    double asp = checktri(argvalhandle.get_brep(0), 
      argvalhandle.get_simpcomp(1),
      os.ostrm(), checko, tol);
    if (returnvalhandle.nargout() > 0)
      returnvalhandle.put_double(0, asp);
    return;
  }
}

#include "gm_entrypoint.cpp"
