# initialization script for Tcl 8.0/Tk8.0 front end to QMG 2.0.




if {![info exists env(QMG_DATA)]} {
    set qmg_data $qmg_library/../data
} else {
    set qmg_data $env(QMG_DATA)
}

set tcl_precision 17
set gm_default_tol 1e-14
set qmg_library2 $qmg_library/../tcl2
set gm_default_axis_color bisque
set gm_default_button_color honeydew
set PI  3.14159265358979
gm_enable_string_rep 1
set gm_brep_type_code brep_v2.0
set gm_simpcomp_type_code mesh_v2.01

if {$argv0 == "qmg_no_windows"} {
    set gm_no_windows 1
}


# This procedure looks for libtkXX.so under Unix or tkXX.dll
# under windows and loads it dynamically.

proc qmg_start_tk {} {
    global qmg_library
    upvar #0 tcl_platform tcl_platform
    upvar #0 tcl_version tcl_version
    upvar #0 env env
    if {[lsearch -exact [array names env] TK_DLL_PATH] < 0} {
	error "Environment variable TK_DLL_PATH not set.\nSee QMG installation instructions in readme file."
    }
    if {$tcl_platform(platform) == "windows"} {
	set ver $tcl_version
	# remove all periods from ver
	regsub -all \\. $ver "" ver2
	load $env(TK_DLL_PATH)/tk$ver2[info sharedlibextension]
    } else {
	load $env(TK_DLL_PATH)/libtk$tcl_version[info sharedlibextension]
    }
}

if {$argc == 0} {
    source $qmg_library/qmg_interactive_init.tcl
} else {
    source $qmg_library/qmg_batch_init.tcl
}
	

# ------------------------------------------------------------------
# Copyright (c) 1999 by Cornell University.  All rights reserved
# See the accompanying file 'Copyright' for authorship information,
# the terms of the license governing this software, and disclaimers
# concerning this software.
# ------------------------------------------------------------------
# This file is part of the QMG software.  
# Version 2.0 of QMG, release date September 3, 1999
# ------------------------------------------------------------------
